package de.intevation.flys.importer;

import java.util.List;

import java.math.BigDecimal;

import org.apache.log4j.Logger;

import org.hibernate.Session;
import org.hibernate.Query;

import de.intevation.flys.model.BedHeightEpoch;
import de.intevation.flys.model.BedHeightEpochValue;


public class ImportBedHeightEpochValue {

    private static final Logger log =
        Logger.getLogger(ImportBedHeightEpochValue.class);


    private BigDecimal station;
    private BigDecimal height;

    private BedHeightEpochValue peer;


    public ImportBedHeightEpochValue() {
    }


    public ImportBedHeightEpochValue(BigDecimal station, BigDecimal height) {
        this.station = station;
        this.height  = height;
    }


    public void storeDependencies(BedHeightEpoch bedHeight) {
        getPeer(bedHeight);
    }


    public BedHeightEpochValue getPeer(BedHeightEpoch bedHeight) {
        if (peer == null) {
            Session session = ImporterSession.getInstance().getDatabaseSession();

            Query query = session.createQuery(
                "from BedHeightEpochValue where " +
                "   bedHeight=:bedHeight and " +
                "   station=:station and " +
                "   height=:height");

            query.setParameter("bedHeight", bedHeight);
            query.setParameter("station", station);
            query.setParameter("height", height);

            List<BedHeightEpochValue> values = query.list();

            if (values.isEmpty()) {
                peer = new BedHeightEpochValue(
                    bedHeight,
                    station,
                    height
                );
                session.save(peer);
            }
            else {
                peer = values.get(0);
            }
        }

        return peer;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
